/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     |                                                 |
|   \\  /    A nd           | Copyright (C) 2016 Ehsan Madadi-Kandjani        |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    `format'      ascii;
    class       dictionary;
    object      blockMeshDict;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// General macros to create cylinder mesh

changecom(//)changequote([,])
define(calc, [esyscmd(perl -e 'use Math::Trig; print ($1)')])
define(VCOUNT, 0)
define(vlabel, [[// ]Vertex $1 = VCOUNT define($1, VCOUNT)define([VCOUNT], incr(VCOUNT))])

define(hex2D, hex ($1b $2b $3b $4b $1t $2t $3t $4t))
define(btQuad, ($1b $2b $2t $1t))
define(topQuad, ($1t $4t $3t $2t))
define(bottomQuad, ($1b $2b $3b $4b))

scale    1;

//****Defined resolution parameters****//

// Number of cells in inner square 
define(Ns, 48)

// Number of cells between inner square and disk boundary 
define(Ni, 16)

// Number of cells between disk and outer boundaries
define(No, 60)

// Number of cells in the bottom cylinder height
define(N1, 40)

// Number of cells in the top cylinder height 
define(N2, 40)

// Number of cells in the disc height  
define(Nd, 8)

//**** Defined grading parameters ****//

// z grading distances  dR1 = 5d
define(dr1, 0.5)
define(dr2, 2.5)
define(dz1, 0.9)
define(dz2, 0.05)

// z grading points fraction  pR2*No = 5*Nd
define(pr1, 1)
define(pr2, 2)
define(pz1, 0.9)
define(pz2, 0.1)

// z grading - works with N1 = N2 = 5*Nd
define(gr,  5.005)
define(gz1, 0.2882)
define(gz2, 3.470)

//**** Defined geometric parameters ****//

// Inner square side half
define(s, 0.5)

// Inner square side curvature
define(sc, 0.6)

// Disc radius
define(rd, 1.00)

// Cylinder radius
define(rc, 4.00)

// Height of bottom cylinder 
define(z1, 0.95)

// Height of the disc
define(zd, 0.1)

// Height of top cylinder 
define(z2, 0.95)

//****Calculated geometric parameters****//

// Base z
define(Zb, 0)

// Top of bottom cylinder
define(Z1, calc(Zb + z1))

// Top of disc 
define(Zd, calc(Z1 + zd))

// Top of top cylinder
define(Z2, calc(Zd + z2))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

define(vert, (x$1$2 y$1$2 $3))
define(evert, (ex$1$2 ey$1$2 $3))

// 45 degree points angle
define(a0, -45)
define(a1, -135)
define(a2, 135)
define(a3, 45)

// Half of 45 degree points angle
define(ea0, 0)
define(ea1, -90)
define(ea2, 180)
define(ea3, 90)

define(ca0, calc(cos((pi/180)*a0)))
define(ca1, calc(cos((pi/180)*a1)))
define(ca2, calc(cos((pi/180)*a2)))
define(ca3, calc(cos((pi/180)*a3)))

define(sa0, calc(sin((pi/180)*a0)))
define(sa1, calc(sin((pi/180)*a1)))
define(sa2, calc(sin((pi/180)*a2)))
define(sa3, calc(sin((pi/180)*a3)))

define(cea0, calc(cos((pi/180)*ea0)))
define(cea1, calc(cos((pi/180)*ea1)))
define(cea2, calc(cos((pi/180)*ea2)))
define(cea3, calc(cos((pi/180)*ea3)))

define(sea0, calc(sin((pi/180)*ea0)))
define(sea1, calc(sin((pi/180)*ea1)))
define(sea2, calc(sin((pi/180)*ea2)))
define(sea3, calc(sin((pi/180)*ea3)))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

// Inner square x and y position

// x
define(x00, s)
define(x01, calc(-1.0*s))
define(x02, calc(-1.0*s))
define(x03, s)

// y
define(y00, calc(-1.0*s))
define(y01, calc(-1.0*s))
define(y02, s)
define(y03, s)

// Disc x and y positions

// x
define(x10, calc(rd*ca0))
define(x11, calc(rd*ca1))
define(x12, calc(rd*ca2))
define(x13, calc(rd*ca3))

// y
define(y10, calc(rd*sa0))
define(y11, calc(rd*sa1))
define(y12, calc(rd*sa2))
define(y13, calc(rd*sa3))

// Cylinder x and y positions

// x
define(x20, calc(rc*ca0))
define(x21, calc(rc*ca1))
define(x22, calc(rc*ca2))
define(x23, calc(rc*ca3))

// y
define(y20, calc(rc*sa0))
define(y21, calc(rc*sa1))
define(y22, calc(rc*sa2))
define(y23, calc(rc*sa3))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

// Inner square x and y position middle curvatures

// x
define(ex00, sc)
define(ex01, 0)
define(ex02, calc(-1.0*sc))
define(ex03, 0)

// y
define(ey00, 0)
define(ey01, calc(-1.0*sc))
define(ey02, 0)
define(ey03, sc)

// Disc x and y positions middle curvatures

// x
define(ex10, calc(rd*cea0))
define(ex11, calc(rd*cea1))
define(ex12, calc(rd*cea2))
define(ex13, calc(rd*cea3))

// y
define(ey10, calc(rd*sea0))
define(ey11, calc(rd*sea1))
define(ey12, calc(rd*sea2))
define(ey13, calc(rd*sea3))

// Cylinder x and y positions middle curvatures

// x
define(ex20, calc(rc*cea0))
define(ex21, calc(rc*cea1))
define(ex22, calc(rc*cea2))
define(ex23, calc(rc*cea3))

// y
define(ey20, calc(rc*sea0))
define(ey21, calc(rc*sea1))
define(ey22, calc(rc*sea2))
define(ey23, calc(rc*sea3))


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

vertices
(
    vert(0, 0, Zb) vlabel(bs0b)      // Bottom cylinder
    vert(0, 1, Zb) vlabel(bs1b)
    vert(0, 2, Zb) vlabel(bs2b)
    vert(0, 3, Zb) vlabel(bs3b)

    vert(1, 0, Zb) vlabel(bd0b)
    vert(1, 1, Zb) vlabel(bd1b)
    vert(1, 2, Zb) vlabel(bd2b)
    vert(1, 3, Zb) vlabel(bd3b)
    
    vert(2, 0, Zb) vlabel(bc0b)
    vert(2, 1, Zb) vlabel(bc1b)
    vert(2, 2, Zb) vlabel(bc2b)
    vert(2, 3, Zb) vlabel(bc3b)

    vert(0, 0, Z1) vlabel(bs0t)
    vert(0, 1, Z1) vlabel(bs1t)
    vert(0, 2, Z1) vlabel(bs2t)
    vert(0, 3, Z1) vlabel(bs3t)
    
    vert(1, 0, Z1) vlabel(bd0t)
    vert(1, 1, Z1) vlabel(bd1t)
    vert(1, 2, Z1) vlabel(bd2t)
    vert(1, 3, Z1) vlabel(bd3t)

    vert(2, 0, Z1) vlabel(bc0t)
    vert(2, 1, Z1) vlabel(bc1t)
    vert(2, 2, Z1) vlabel(bc2t)
    vert(2, 3, Z1) vlabel(bc3t)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    vert(0, 0, Zd) vlabel(ts0b)      // Top cylinder
    vert(0, 1, Zd) vlabel(ts1b)
    vert(0, 2, Zd) vlabel(ts2b)
    vert(0, 3, Zd) vlabel(ts3b)

    vert(1, 0, Zd) vlabel(td0b)
    vert(1, 1, Zd) vlabel(td1b)
    vert(1, 2, Zd) vlabel(td2b)
    vert(1, 3, Zd) vlabel(td3b)

    vert(2, 0, Zd) vlabel(tc0b)
    vert(2, 1, Zd) vlabel(tc1b)
    vert(2, 2, Zd) vlabel(tc2b)
    vert(2, 3, Zd) vlabel(tc3b)

    vert(0, 0, Z2) vlabel(ts0t)
    vert(0, 1, Z2) vlabel(ts1t)
    vert(0, 2, Z2) vlabel(ts2t)
    vert(0, 3, Z2) vlabel(ts3t)

    vert(1, 0, Z2) vlabel(td0t)
    vert(1, 1, Z2) vlabel(td1t)
    vert(1, 2, Z2) vlabel(td2t)
    vert(1, 3, Z2) vlabel(td3t)

    vert(2, 0, Z2) vlabel(tc0t)
    vert(2, 1, Z2) vlabel(tc1t)
    vert(2, 2, Z2) vlabel(tc2t)
    vert(2, 3, Z2) vlabel(tc3t)

);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

blocks
(
    //block0                        // Bottom cylinder
    hex2D(bs1, bs0, bs3, bs2)
    bottomCylinder
    (Ns Ns N1)
    simpleGrading  
    (
        1
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )

    //block1
    hex2D(bs0, bd0, bd3, bs3)
    bottomCylinder
    (Ni Ns N1)
    simpleGrading
    (
        1
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )

    //block2
    hex2D(bs3, bd3, bd2, bs2)
    bottomCylinder
    (Ni Ns N1)
    simpleGrading
    (
        1
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )

    //block3
    hex2D(bs2, bd2, bd1, bs1)
    bottomCylinder
    (Ni Ns N1)
    simpleGrading
    (
        1
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )

    //block4
    hex2D(bs1, bd1, bd0, bs0)
    bottomCylinder
    (Ni Ns N1)
    simpleGrading
    (
        1
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )

    //block5
    hex2D(bd0, bc0, bc3, bd3)
    bottomCylinder
    (No Ns N1)
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )
    
    //block6
    hex2D(bd3, bc3, bc2, bd2)
    bottomCylinder
    (No Ns N1)
    simpleGrading
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )
    
    //block7
    hex2D(bd2, bc2, bc1, bd1)
    bottomCylinder
    (No Ns N1)
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )
    
    //block8
    hex2D(bd1, bc1, bc0, bd0)
    bottomCylinder
    (No Ns N1)
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 1)
        )
    )

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    //block9                       // Disc ring
    hex (bd0t bc0t bc3t bd3t td0b tc0b tc3b td3b)
    discRing
    (No Ns Nd )
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        1
    )

    //block10
    hex (bd3t bc3t bc2t bd2t td3b tc3b tc2b td2b)
    discRing
    (No Ns Nd )
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        1
    )


    //block11
    hex (bd2t bc2t bc1t bd1t td2b tc2b tc1b td1b)
    discRing
    (No Ns Nd )
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        1
    )


    //block12
    hex (bd1t bc1t bc0t bd0t td1b tc1b tc0b td0b)
    discRing
    (No Ns Nd )
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        1
    )


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    //block13                      // Top cylinder
    hex2D(ts1, ts0, ts3, ts2)
    topCylinder         
    (Ns Ns N2 )
    simpleGrading 
    (
        1
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block14
    hex2D(ts0, td0, td3, ts3)
    topCylinder
    (Ni Ns N2 )
    simpleGrading
    (
        1
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block15
    hex2D(ts3, td3, td2, ts2)
    topCylinder
    (Ni Ns N2 )
    simpleGrading
    (
        1
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block16
    hex2D(ts2, td2, td1, ts1)
    topCylinder
    (Ni Ns N2 )
    simpleGrading
    (
        1
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block17
    hex2D(ts1, td1, td0, ts0)
    topCylinder
    (Ni Ns N2 )
    simpleGrading
    (
        1
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block18
    hex2D(td0, tc0, tc3, td3)
    topCylinder        
    (No Ns N2 )
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block19
    hex2D(td3, tc3, tc2, td2)
    topCylinder        
    (No Ns N2 )
    simpleGrading
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block20
    hex2D(td2, tc2, tc1, td1)
    topCylinder        
    (No Ns N2 )
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

    //block21
    hex2D(td1, tc1, tc0, td0)
    topCylinder        
    (No Ns N2 )
    simpleGrading 
    (
        (
            (dr1 pr1 1)
            (dr2 pr2 gr)
        )
        1
        (
            (dz2 pz2 1)
            (dz1 pz1 gz2)
        )
    )

);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

edges
(
                                    //  Z=Zb            
    arc bs3b bs0b evert(0, 0, Zb)
    arc bs0b bs1b evert(0, 1, Zb)
    arc bs1b bs2b evert(0, 2, Zb)
    arc bs2b bs3b evert(0, 3, Zb)

    arc bd3b bd0b evert(1, 0, Zb)
    arc bd0b bd1b evert(1, 1, Zb)
    arc bd1b bd2b evert(1, 2, Zb)
    arc bd2b bd3b evert(1, 3, Zb)
    
    arc bc3b bc0b evert(2, 0, Zb)
    arc bc0b bc1b evert(2, 1, Zb)
    arc bc1b bc2b evert(2, 2, Zb)
    arc bc2b bc3b evert(2, 3, Zb)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //    
                                    //  Z=Z1
    arc bs3t bs0t evert(0, 0, Z1)
    arc bs0t bs1t evert(0, 1, Z1)
    arc bs1t bs2t evert(0, 2, Z1)
    arc bs2t bs3t evert(0, 3, Z1)
    
    arc bd3t bd0t evert(1, 0, Z1)
    arc bd0t bd1t evert(1, 1, Z1)
    arc bd1t bd2t evert(1, 2, Z1)
    arc bd2t bd3t evert(1, 3, Z1)

    arc bc3t bc0t evert(2, 0, Z1)
    arc bc0t bc1t evert(2, 1, Z1)
    arc bc1t bc2t evert(2, 2, Z1)
    arc bc2t bc3t evert(2, 3, Z1)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

                                    //  Z=Zd
    arc ts3b ts0b evert(0, 0, Zd)
    arc ts0b ts1b evert(0, 1, Zd)
    arc ts1b ts2b evert(0, 2, Zd)
    arc ts2b ts3b evert(0, 3, Zd)

    arc td3b td0b evert(1, 0, Zd)
    arc td0b td1b evert(1, 1, Zd)
    arc td1b td2b evert(1, 2, Zd)
    arc td2b td3b evert(1, 3, Zd)

    arc tc3b tc0b evert(2, 0, Zd)
    arc tc0b tc1b evert(2, 1, Zd)
    arc tc1b tc2b evert(2, 2, Zd)
    arc tc2b tc3b evert(2, 3, Zd)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

                                    //  Z=Z2
    arc ts3t ts0t evert(0, 0, Z2)
    arc ts0t ts1t evert(0, 1, Z2)
    arc ts1t ts2t evert(0, 2, Z2)
    arc ts2t ts3t evert(0, 3, Z2)

    arc td3t td0t evert(1, 0, Z2)
    arc td0t td1t evert(1, 1, Z2)
    arc td1t td2t evert(1, 2, Z2)
    arc td2t td3t evert(1, 3, Z2)

    arc tc3t tc0t evert(2, 0, Z2)
    arc tc0t tc1t evert(2, 1, Z2)
    arc tc1t tc2t evert(2, 2, Z2)
    arc tc2t tc3t evert(2, 3, Z2)
);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

patches
(
    patch cylinderWall
    (
        (bc3b bc3t bc0t bc0b)
        (bc2b bc2t bc3t bc3b)
        (bc1b bc1t bc2t bc2b)
        (bc0b bc0t bc1t bc1b)

        (bc3t tc3b tc0b bc0t)
        (bc2t tc2b tc3b bc3t)
        (bc1t tc1b tc2b bc2t)
        (bc0t tc0b tc1b bc1t)

        (tc3b tc3t tc0t tc0b)
        (tc0b tc0t tc1t tc1b)
        (tc1b tc1t tc2t tc2b)
        (tc2b tc2t tc3t tc3b)
    )

    patch cylinderBtm
    (
        bottomQuad(bs3, bs0, bs1, bs2)
        bottomQuad(bs0, bd0, bd3, bs3)
        bottomQuad(bs3, bd3, bd2, bs2)
        bottomQuad(bs2, bd2, bd1, bs1)
        bottomQuad(bs1, bd1, bd0, bs0)
        bottomQuad(bd0, bc0, bc3, bd3)
        bottomQuad(bd3, bc3, bc2, bd2)
        bottomQuad(bd2, bc2, bc1, bd1)
        bottomQuad(bd1, bc1, bc0, bd0)
    )
    
    patch cylinderTop
    (
        topQuad(ts3, ts0, ts1, ts2)
        topQuad(ts0, td0, td3, ts3)
        topQuad(ts3, td3, td2, ts2)
        topQuad(ts2, td2, td1, ts1)
        topQuad(ts1, td1, td0, ts0)
        topQuad(td0, tc0, tc3, td3)
        topQuad(td3, tc3, tc2, td2)
        topQuad(td2, tc2, tc1, td1)
        topQuad(td1, tc1, tc0, td0)
    )

    patch discWall
    (
        (bd3t td3b td0b bd0t)
        (bd0t td0b td1b bd1t)
        (bd1t td1b td2b bd2t)
        (bd2t td2b td3b bd3t)
    )

    patch discTop
    (
        (ts3b ts0b ts1b ts2b)
        (td3b ts3b ts0b td0b)
        (td0b ts0b ts1b td1b)
        (td1b ts1b ts2b td2b)
        (td2b ts2b ts3b td3b)
    )

    patch discBtm
    (
        (bs3t bs0t bs1t bs2t)
        (bd3t bs3t bs0t bd0t)
        (bd0t bs0t bs1t bd1t)
        (bd1t bs1t bs2t bd2t)
        (bd2t bs2t bs3t bd3t)
    )
);

mergePatchPairs
(
);
