/*--------------------------------*- C++ -*----------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     |                                                 |
|   \\  /    A nd           | Copyright (C) 2016 Ehsan Madadi-Kandjani        |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    `format'      ascii;
    class       dictionary;
    object      blockMeshDict;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// General macros to create cylinder mesh

changecom(//)changequote([,])
define(calc, [esyscmd(perl -e 'use Math::Trig; print ($1)')])
define(VCOUNT, 0)
define(vlabel, [[// ]Vertex $1 = VCOUNT define($1, VCOUNT)define([VCOUNT], incr(VCOUNT))])

define(hex2D, hex ($1b $2b $3b $4b $1t $2t $3t $4t))
define(btQuad, ($1b $2b $2t $1t))
define(topQuad, ($1t $4t $3t $2t))
define(bottomQuad, ($1b $2b $3b $4b))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

scale 1;

//****Defined resolution parameters****//

// Number of cells at inner square 
define(Ns, 48)

// Number of cells between inner square and circle 
define(Nx, 150)

// Number of cells in the bottom cylinder height
define(N1, 48)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

//**** Defined grading parameters ****//

// z grading distances  dR1 = 5d
define(dr1, 1)
define(dr2, 5)
define(dz1, 1)
define(dz2, 1)

// z grading points fraction  pR2*No = 5*Nd
define(pr1, 4)
define(pr2,11)
define(pz1, 1)
define(pz2, 1)

// z grading - works with N1 = N2 = 5*Nd
define(gr1, 4.7)
define(gr2, 0.857)
define(gz1, 4)
define(gz2, 0.25)

//**** Defined geometric parameters ****//

// Inner square side half
define(s, 0.35)

// Inner square side curvature
define(sc, 0.4)

// Cylinder radius
define(rc, 10)

// Height of cylinder 
define(z1, 2)

//****Calculated geometric parameters****//

// Base z
define(Zb, 0)

// Top of bottom cylinder
define(Z1, calc(Zb + z1))

define(vert, (x$1$2 y$1$2 $3))
define(evert, (ex$1$2 ey$1$2 $3))

// 45 degree points angle
define(a0, -45)
define(a1, -135)
define(a2, 135)
define(a3, 45)

// Half of 45 degree points angle
define(ea0, 0)
define(ea1, -90)
define(ea2, 180)
define(ea3, 90)

define(ca0, calc(cos((pi/180)*a0)))
define(ca1, calc(cos((pi/180)*a1)))
define(ca2, calc(cos((pi/180)*a2)))
define(ca3, calc(cos((pi/180)*a3)))

define(sa0, calc(sin((pi/180)*a0)))
define(sa1, calc(sin((pi/180)*a1)))
define(sa2, calc(sin((pi/180)*a2)))
define(sa3, calc(sin((pi/180)*a3)))

define(cea0, calc(cos((pi/180)*ea0)))
define(cea1, calc(cos((pi/180)*ea1)))
define(cea2, calc(cos((pi/180)*ea2)))
define(cea3, calc(cos((pi/180)*ea3)))

define(sea0, calc(sin((pi/180)*ea0)))
define(sea1, calc(sin((pi/180)*ea1)))
define(sea2, calc(sin((pi/180)*ea2)))
define(sea3, calc(sin((pi/180)*ea3)))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

// Inner square x and y position

// x
define(x00, s)
define(x01, calc(-1.0*s))
define(x02, calc(-1.0*s))
define(x03, s)

// y
define(y00, calc(-1.0*s))
define(y01, calc(-1.0*s))
define(y02, s)
define(y03, s)

// Cylinder x and y positions

// x
define(x20, calc(rc*ca0))
define(x21, calc(rc*ca1))
define(x22, calc(rc*ca2))
define(x23, calc(rc*ca3))

// y
define(y20, calc(rc*sa0))
define(y21, calc(rc*sa1))
define(y22, calc(rc*sa2))
define(y23, calc(rc*sa3))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

// Inner square x and y position middle curvatures

// x
define(ex00, sc)
define(ex01, 0)
define(ex02, calc(-1.0*sc))
define(ex03, 0)

// y
define(ey00, 0)
define(ey01, calc(-1.0*sc))
define(ey02, 0)
define(ey03, sc)

// Cylinder x and y positions middle curvatures

// x
define(ex20, calc(rc*cea0))
define(ex21, calc(rc*cea1))
define(ex22, calc(rc*cea2))
define(ex23, calc(rc*cea3))

// y
define(ey20, calc(rc*sea0))
define(ey21, calc(rc*sea1))
define(ey22, calc(rc*sea2))
define(ey23, calc(rc*sea3))


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

vertices
(
    vert(0, 0, Zb) vlabel(bs0b)      // Bottom cylinder
    vert(0, 1, Zb) vlabel(bs1b)
    vert(0, 2, Zb) vlabel(bs2b)
    vert(0, 3, Zb) vlabel(bs3b)

    vert(2, 0, Zb) vlabel(bc0b)
    vert(2, 1, Zb) vlabel(bc1b)
    vert(2, 2, Zb) vlabel(bc2b)
    vert(2, 3, Zb) vlabel(bc3b)

    vert(0, 0, Z1) vlabel(bs0t)
    vert(0, 1, Z1) vlabel(bs1t)
    vert(0, 2, Z1) vlabel(bs2t)
    vert(0, 3, Z1) vlabel(bs3t)
    
    vert(2, 0, Z1) vlabel(bc0t)
    vert(2, 1, Z1) vlabel(bc1t)
    vert(2, 2, Z1) vlabel(bc2t)
    vert(2, 3, Z1) vlabel(bc3t)
);
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

blocks
(
    //block0                        // Bottom cylinder
    hex2D(bs1, bs0, bs3, bs2)
    bottomCylinder
    (Ns Ns N1 )
    simpleGrading  
    (
        1
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 gz2)
        )
    )

    //block1
    hex2D(bs0, bc0, bc3, bs3)
    bottomCylinder
    (Nx Ns N1 )
    simpleGrading
    (
        (
            (dr1 pr1 gr1)
            (dr2 pr2 gr2)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 gz2)
        )
    )

    //block2
    hex2D(bs3, bc3, bc2, bs2)
    bottomCylinder
    (Nx Ns N1 )
    simpleGrading
    (
        (
            (dr1 pr1 gr1)
            (dr2 pr2 gr2)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 gz2)
        )
    )

    //block3
    hex2D(bs2, bc2, bc1, bs1)
    bottomCylinder
    (Nx Ns N1 )
    simpleGrading
    (
        (
            (dr1 pr1 gr1)
            (dr2 pr2 gr2)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 gz2)
        )
    )

    //block4
    hex2D(bs1, bc1, bc0, bs0)
    bottomCylinder
    (Nx Ns N1 )
    simpleGrading
    (
        (
            (dr1 pr1 gr1)
            (dr2 pr2 gr2)
        )
        1
        (
            (dz1 pz1 gz1)
            (dz2 pz2 gz2)
        )
    )
);
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //


edges
(
                                    //  Z=Zb            
    arc bs3b bs0b evert(0, 0, Zb)
    arc bs0b bs1b evert(0, 1, Zb)
    arc bs1b bs2b evert(0, 2, Zb)
    arc bs2b bs3b evert(0, 3, Zb)

    arc bc3b bc0b evert(2, 0, Zb)
    arc bc0b bc1b evert(2, 1, Zb)
    arc bc1b bc2b evert(2, 2, Zb)
    arc bc2b bc3b evert(2, 3, Zb)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //    
                                    //  Z=Z1
    arc bs3t bs0t evert(0, 0, Z1)
    arc bs0t bs1t evert(0, 1, Z1)
    arc bs1t bs2t evert(0, 2, Z1)
    arc bs2t bs3t evert(0, 3, Z1)
    
    arc bc3t bc0t evert(2, 0, Z1)
    arc bc0t bc1t evert(2, 1, Z1)
    arc bc1t bc2t evert(2, 2, Z1)
    arc bc2t bc3t evert(2, 3, Z1)
);
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

patches
(
    patch cylinderWall
    (
        (bc3b bc3t bc0t bc0b)
        (bc2b bc2t bc3t bc3b)
        (bc1b bc1t bc2t bc2b)
        (bc0b bc0t bc1t bc1b)

    )

    patch cylinderBtm
    (
        bottomQuad(bs3, bs0, bs1, bs2)
        bottomQuad(bs0, bc0, bc3, bs3)
        bottomQuad(bs3, bc3, bc2, bs2)
        bottomQuad(bs2, bc2, bc1, bs1)
        bottomQuad(bs1, bc1, bc0, bs0)
    )
    
    patch cylinderTop
    (
        topQuad(bs3, bs0, bs1, bs2)
        topQuad(bs0, bc0, bc3, bs3)
        topQuad(bs3, bc3, bc2, bs2)
        topQuad(bs2, bc2, bc1, bs1)
        topQuad(bs1, bc1, bc0, bs0)
    )

);

mergePatchPairs
(
);
